namespace Deskcandy.Candy {
  public class XScreensaverConfig : GXml.Element {
    [Description(nick="::name")]
    public string name { get; set; }

    [Description(nick="::_label")]
    public string label { get; set; }

    [Description(nick="::gl")]
    public string gl { get; set; }

    public string description { get; set; }
    public string arg { get; set;}

    public async XScreensaverConfig(File file) throws Error {
      this.initialize("screensaver");

      yield this.read_from_file_async(file);

      this.arg = this.get_child("command").get_attribute("arg");
      this.description = this.get_child("_description").text_content.strip();
      debug(@"Loaded $this");
    }

    public string to_string() {
      return @"Screensaver: $(this.label) [$(this.name)]";
    }

    public Gtk.Box get_widgets() {
      var root = new Gtk.Box(Gtk.Orientation.VERTICAL, 5);
      this.get_widgets_internal(root, this);
      return root;
    }

    private void get_widgets_internal(Gtk.Box parent, GXml.Element elem) {
      foreach (var child in elem.children) {
        var child_container = parent;
        if (child.has_attribute("_label") && child.tag_name != "boolean") {
          child_container = new Gtk.Box(Gtk.Orientation.VERTICAL, 5);

          // todo: css?
          var label = new Gtk.Label("");
          label.xalign = 0.0f;
          label.use_markup = true;
          label.set_markup(@"<i>$(child.get_attribute("_label"))</i>");
          child_container.pack_start(label, false);
          parent.pack_start(child_container);
        }

        switch (child.tag_name) {
          case "hgroup":
            var hbox = new Gtk.Box(Gtk.Orientation.HORIZONTAL, 5);
            this.get_widgets_internal(hbox, (GXml.Element)child);
            child_container.pack_start(hbox, false);
            break;

          case "vgroup":
            var vbox = new Gtk.Box(Gtk.Orientation.VERTICAL, 5);
            this.get_widgets_internal(vbox, (GXml.Element)child);
            child_container.pack_start(vbox);
            break;

          case "boolean":
            var widget = new Gtk.CheckButton.with_label(child.get_attribute("_label"));
            child_container.pack_start(widget);
            break;

          case "number":
            Gtk.Widget widget = null;

            var adjust = new Gtk.Adjustment(
              double.parse(child.get_attribute("default")),
              double.parse(child.get_attribute("low")),
              double.parse(child.get_attribute("high")),
              0.01, 0.0, 0.0);

            switch (child.get_attribute("type")) {
              case "spinbutton":
                widget = new Gtk.SpinButton(adjust, 0.0, 1);
                break;
              case "slider":
                var scale = new Gtk.Scale(Gtk.Orientation.HORIZONTAL, adjust);
                scale.value_pos = Gtk.PositionType.LEFT;
                widget = scale;
                break;
              default:
                warning(@"unhandled number type $(child.get_attribute("type"))");
                continue;
            }

            child_container.pack_start(widget);

            break;

          case "select":
            var model = new Gtk.ListStore(2, Type.STRING, Type.STRING);
            Gtk.TreeIter? iter = null;
            foreach (var option in child.get_elements_by_tag_name("option"))
            {
              model.insert_with_values(out iter, -1, 0, option.get_attribute("id"), 1, option.get_attribute("_label"));
            }

            var select = new Gtk.ComboBox.with_model(model);
            select.id_column = 0;
            var select_text = new Gtk.CellRendererText();
            select.pack_start(select_text, true);
            select.add_attribute(select_text, "text", 1);
            select.active = 0;
            child_container.pack_start(select);
            break;
          case "file":
            var file_chooser = new Gtk.FileChooserButton(child.get_attribute("_label"), Gtk.FileChooserAction.OPEN);
            child_container.pack_start(file_chooser);
            break;
        }
      }
    }
  }
}
